/* $Id: db.c 125 2023-08-06 19:30:08Z nishi $ */
#include <swinehub/db.h>

#include <swinehub.h>

#include <stdlib.h>
#include <fcntl.h>
#include <sys/stat.h>
#include <errno.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>

DBM* swinehub_token_db = NULL;
DBM* swinehub_user_db = NULL;

#define SLEEP_A_BIT usleep(100000)

void swinehub_db_close(){
	if(swinehub_user_db != NULL){
		dbm_close(swinehub_user_db);
		remove(SWINEHUB_TOKEN_DB ".lock");
	}
	if(swinehub_token_db != NULL){
		dbm_close(swinehub_token_db);
		remove(SWINEHUB_USER_DB ".lock");
	}
	swinehub_user_db = NULL;
	swinehub_token_db = NULL;
}

int swinehub_db_init(){
	if(swinehub_token_db != NULL){
		dbm_close(swinehub_token_db);
		swinehub_token_db = NULL;
	}
	if(swinehub_user_db != NULL){
		dbm_close(swinehub_user_db);
		swinehub_user_db = NULL;
	}
retry_db:
	if(access(SWINEHUB_TOKEN_DB ".lock", F_OK) == 0){
		FILE* f = fopen(SWINEHUB_TOKEN_DB ".lock", "r");
		struct stat s;
		stat(SWINEHUB_TOKEN_DB ".lock", &s);
		char* buf = malloc(s.st_size + 1);
		buf[s.st_size] = 0;
		fread(buf, s.st_size, 1, f);
		int pid = atoi(buf);
		fclose(f);
		free(buf);
		if(pid != getpid() && kill(pid, 0) == 0){
			SLEEP_A_BIT;
			goto retry_db;
		}
		remove(SWINEHUB_TOKEN_DB ".lock");
	}
	if(access(SWINEHUB_USER_DB ".lock", F_OK) == 0){
		FILE* f = fopen(SWINEHUB_USER_DB ".lock", "r");
		struct stat s;
		stat(SWINEHUB_USER_DB ".lock", &s);
		char* buf = malloc(s.st_size + 1);
		buf[s.st_size] = 0;
		fread(buf, s.st_size, 1, f);
		int pid = atoi(buf);
		fclose(f);
		free(buf);
		if(pid != getpid() && kill(pid, 0) == 0){
			SLEEP_A_BIT;
			goto retry_db;
		}
		remove(SWINEHUB_USER_DB ".lock");
	}
	FILE* f = fopen(SWINEHUB_TOKEN_DB ".lock", "w");
	fprintf(f, "%d", getpid());
	fclose(f);
	f = fopen(SWINEHUB_USER_DB ".lock", "w");
	fprintf(f, "%d", getpid());
	fclose(f);
	if(swinehub_token_db == NULL){
		swinehub_token_db = dbm_open(
			SWINEHUB_TOKEN_DB,
			O_RDWR | O_CREAT,
			S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH
		);
	}
	if(swinehub_user_db == NULL){
		swinehub_user_db = dbm_open(
			SWINEHUB_USER_DB,
			O_RDWR | O_CREAT,
			S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH
		);
	}
	if(swinehub_token_db == NULL || swinehub_user_db == NULL){
		SLEEP_A_BIT;
		goto retry_db;
	}
	return 0;
}
